#include <stdlib.h>
#include "list.h"

void list_init(linked_list* list)
{
    list->n_elements = 0;
    
    //A dummy element to check if we have reached the end of the list
    list->last_element = malloc(sizeof(list_element));
    list->last_element->content = NULL;
    list->last_element->next = NULL;
    list->last_element->prev = NULL;
    
    list->first_element = list->last_element;
    list->current_element = list->last_element;
}

void list_add_element(linked_list *list, void* content)
{
    list_element *new_element = malloc(sizeof(list_element));
    new_element->content = content;
    new_element->next = list->last_element;

    /* Check if list is empty */
    if (list->n_elements == 0)
    {
        list->first_element = new_element;
        list->current_element = new_element;
        new_element->prev = NULL;
    }
    else {
        new_element->prev = list->last_element->prev;
        list->last_element->prev->next = new_element;
    }
    
    list->last_element->prev = new_element;
    
    list->n_elements++;
}

void list_add_element_before_current(linked_list *list, void* content)
{
    list_element *new_element = malloc(sizeof(list_element));
    new_element->content = content;
    new_element->next = list->current_element;

    /* Check if list is empty */
    if (list->n_elements == 0)
    {
        list->first_element = new_element;
        list->current_element = new_element;
        new_element->prev = NULL;
        list->last_element->prev = new_element;
    }
    else {
        new_element->prev = list->current_element->prev;
        list->current_element->prev = new_element;
        /* Check if the new element is the first element in the list */
        if (new_element->prev == NULL)
        {
            list->first_element = new_element;
        }
        else{
            new_element->prev->next = new_element;
        }
    }
    
    list->n_elements++;
}

void* list_get_current(linked_list *list)
{
    if (list == NULL)
    {
        return NULL;
    }
    if (list->current_element == list->last_element)
    {
        return NULL;
    }
    return list->current_element->content;
}

void* list_get_next(linked_list *list)
{
    void *return_me;
    if (list->n_elements == 0)
    {
        return NULL;
    }
    if (list->current_element == list->last_element)
    {
        list->current_element = list->first_element;
        return NULL;
    }
    return_me = list->current_element->content;
    list->current_element = list->current_element->next;
    return return_me;
}

void* list_remove_current(linked_list *list)
{
    list_element* delete_me = list->current_element;
    if (delete_me == list->last_element)
    {
        return NULL;
    }
    void* return_me = delete_me->content;

    delete_me->next->prev = delete_me->prev;
    list->current_element = delete_me->next;
    
    //list->current_element = list->first_element;
    
    if (delete_me->prev != NULL) 
    {
        delete_me->prev->next = delete_me->next;
    }
    else {
        // This is the first element
        list->first_element = delete_me->next;
    }

    free(delete_me);

    list->n_elements--;

    return return_me;
}

void list_remove_by_content(linked_list *list, void* content)
{
    list_element *current = list->first_element;
    while(current != list->last_element)
    {
        if (current->content == content)
        {
            current->next->prev = current->prev;
            
            
            
            if (current->prev != NULL) 
            {
                current->prev->next = current->next;
            }
            else {
                // This is the first element
                list->first_element = current->next;
            }

            if (current == list->current_element)
            {
                list->current_element = current->next;
            }

            free(current);

            list->n_elements--;
        }
        current = current->next;
    }
}

void list_destroy(linked_list *list)
{
    if (list == NULL)
    {
        return;
    }
    list_element *delete_me = list->first_element;
    list_element *next;
    while(delete_me != NULL)
    {
        next = delete_me->next;
        free(delete_me);
        delete_me = next;
    }
}

