#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <pthread.h>
#include "task.h"
#include "sched.h"


pthread_mutex_t mutex;

void high_crit_thread_func(void *args)
{
    printf("High Crit\n");
    usleep(400000);
}

unsigned int high_crit_estimator(void *args)
{
    return 500;
}

void hough_preproc(void *args, void *result)
{
    unsigned int n_edgepixels = rand()%40000;
    pthread_mutex_lock(&mutex);
    *((unsigned int*)args) = n_edgepixels;
    pthread_mutex_unlock(&mutex);
    sleep(0.05);
}

void hough_transform_var_A(void *args)
{
    unsigned int sleep_time;
    pthread_mutex_lock(&mutex);
    sleep_time = (*((unsigned int*)args))*3.0/3500 + 50;
    pthread_mutex_unlock(&mutex);
    printf("Hough A, %d\n", sleep_time);
    usleep(sleep_time);
}

void hough_transform_var_B(void *args)
{
    unsigned int sleep_time;
    pthread_mutex_lock(&mutex);
    sleep_time = (*((unsigned int*)args))*1.0/3500 + 100;
    pthread_mutex_unlock(&mutex);
    printf("Hough B\n");
    usleep(sleep_time);
}

unsigned int hough_wcet_estimator_A(void *args)
{
    unsigned int wcet;
    pthread_mutex_lock(&mutex);
    wcet = (*((unsigned int*)args))*3.0/3500 + 100;
    pthread_mutex_unlock(&mutex);
    printf("wcet, %d\n", wcet);
    return wcet;
}

unsigned int hough_wcet_estimator_B(void *args)
{
    unsigned int wcet;
    pthread_mutex_lock(&mutex);
    wcet = (*((unsigned int*)args))*1.0/3500 + 150;
    pthread_mutex_unlock(&mutex);
    return wcet;
}

int main(int argc, char *argv[])
{
    
    task_tcb hough_task, high_crit_task;
    task_variant hough_var_A, hough_var_B, high_crit_var;

    pthread_mutex_init(&mutex, NULL);

    srand(time(NULL));

    void *wcet_param = malloc(sizeof(unsigned int));

    task_init(&hough_task, "HOUGH", 1, 100, 1, &hough_preproc, wcet_param);
    task_variant_init(&hough_var_A, &hough_transform_var_A, wcet_param, &hough_wcet_estimator_A, wcet_param);
    task_variant_init(&hough_var_B, &hough_transform_var_B, wcet_param, &hough_wcet_estimator_B, wcet_param);
    task_add_variant(&hough_task, &hough_var_A);
    task_add_variant(&hough_task, &hough_var_B);

    task_init(&high_crit_task, "HIGH_CRIT", 2, 100, 2, NULL, NULL);
    task_variant_init(&high_crit_var, &high_crit_thread_func, NULL, &high_crit_estimator, NULL);

    sched_init();

    sched_register_task(&high_crit_task, 1000);
    sched_register_task(&hough_task, 1000);

    sched_start();

    return 0;
}
