#ifndef TASK_H_WJIBS0KT
#define TASK_H_WJIBS0KT

#include "list.h"
#include <pthread.h>
#include <sys/types.h>

#define TASK_MAX_NAME_SIZE 80

typedef struct _task_variant
{
    void (*task_routine)(void *);
    void *task_routine_param;
    unsigned int (*wcet_estimator)(void *);
    void *estimator_param;

} task_variant;

typedef struct _task_shared_data
{
    pid_t pid;
    int finished;
    pthread_mutex_t mutex;
    void* preproc_results;
    //unsigned int preproc_results;

} task_shared_data;

typedef struct _task_tcb
{
    linked_list *task_variants;
    unsigned int n_task_variants;
    unsigned int priority;
    unsigned int deadline;
    unsigned int criticality_level;
    char name[TASK_MAX_NAME_SIZE + 1];
    task_shared_data *shared;
    void (*estimator_preproc)(void* args, void* result);
    void *preproc_param;
} task_tcb;


int task_init(task_tcb *new_task, char *name, unsigned int priority, unsigned int deadline, unsigned int criticality_level, void (*estimator_preproc)(void* args, void* result), void *preproc_param);

int task_add_variant(task_tcb *task, task_variant *task_var);

int task_variant_init(task_variant *task_var, void (*task_routine)(void *), void *task_routine_param, 
                      unsigned int (*wcet_estimator)(void *), void *estimator_param);

unsigned int task_get_wcet(task_tcb *task);

int task_use_next_variant(task_tcb *task);

int task_start(task_tcb *task);

int task_get_finished(task_tcb *task);

int task_set_finished(task_tcb *task, int new_value);

int task_suspend(task_tcb *task);

int task_resume(task_tcb *task);

int task_destroy(task_tcb *task);

#endif /* end of include guard: TASK_H_WJIBS0KT */
